
;**************************************************************************
;*                           M41T00CAP Clock                              *
;*                                                                        *
;*   MSSP2 is used as interface for the TM1637                            *
;**************************************************************************
;**************************************************************************
;*          v 1.08 - 31.12.2018                                           *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC16F1827 used                                       *
;*          OSC.......: Internal osc. used     POWER.....:   5V extern.   *
;**************************************************************************

 INCLUDE "P16F1827.INC"


_U001			     EQU     B'00100110000100'		;see Page 48 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet

	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002

;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

v_pos0		EQU	H'24'		;left side
v_pos1		EQU	H'25'
v_pos2		EQU	H'26'
v_pos3		EQU	H'27'		;right side
;--------------------------------------------------------------------------
v_cnt1		EQU	H'2E'
v_cnt2		EQU	H'2F'
v_cnt3		EQU	H'30'
v_cnt4		EQU	H'31'
v_cnt5		EQU	H'32'
v_cnt6		EQU	H'33'

v_rtc_sec	EQU	H'34'
v_rtc_min	EQU	H'35'
v_rtc_hrs	EQU	H'36'

v_lsb		EQU	H'37'
v_msb		EQU	H'38'
v_value		EQU	H'39'

v_blink		EQU	H'3A'
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************
;
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; initialization
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

init_main	clrf	BSR
		clrf	INTCON		;disable interrupts

		banksel	OPTION_REG	;BANK 1
		movlw	B'00111111'
		movwf	OPTION_REG	;see datasheet Page 176

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA
		clrf	ANSELB

		movlw	H'01'
		movwf	BSR		;select BANK 1
		movlw	B'11111111'
		movwf	TRISA
		movlw	B'11111111'
		movwf	TRISB

		banksel	OSCCON
		movlw	B'01101000'
		movwf	OSCCON		;set 4 MHz internal clock (bit 6:3 = 1101) - Datasheet Page 65

		banksel	WPUB
		clrf	WPUA
		movlw	B'11111111'
		movwf	WPUB

		clrf	BSR		;select BANK 0
;------------------------------------------------------------------------------
		goto	m_application
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
dr4		call	dr2
		call	dr2
		return
;-----------------------------------
dr2		call	d2
		return
;-----------------------------------
;-----------------------------------
d1		movlw	D'050'
		movwf	TIMER1
d1_1		movlw	D'255'
		movwf	TIMER2
d11_loop	decfsz	TIMER2,F
		goto	d11_loop
		decfsz	TIMER1,F
		goto	d1_1
		return
;-----------------------------------
d2		movlw	D'255'
		movwf	TIMER1
d2_1		movlw	D'255'
		movwf	TIMER2
d21_loop	decfsz	TIMER2,F
		goto	d21_loop
		decfsz	TIMER1,F
		goto	d2_1
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
 INCLUDE "90_I2C_comm_16F1827-2xI2C.inc"
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;(A) - Initialization
m_application	movlw	H'10'
		movwf	v_pos0
		movwf	v_pos1
		movwf	v_pos2
		movwf	v_pos3

		call	i2c_init
;--------------------------------------
		call	display_data
		call	d2
		clrf	v_blink
;--------------------------------------
;switch on the display ----------------
		call	i2c_start
		movlw	H'F1'
		call	i2c_send
		call	i2c_stop
;--------------------------------------
;--------------------------------------
;(B) - Application Main Loop
main_loop	nop

		call	rtc_i2c_read		;(1) - read the time information from M41T00CAP

		call	translate_clk		;(2.1) - prepare display data - part 1
		call	translate_chr2		;(2.2) - prepare display data - part 2
		call	display_data

		call	d1

		btfss	PORTB,D'000'
		goto	clk_set_up

		incf	v_blink,1
		goto	main_loop

;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
translate_chr2	movf	v_cnt1,0
		movwf	v_pos0

		movf	v_cnt2,0
		movwf	v_pos1

		movf	v_cnt3,0
		movwf	v_pos2

		movf	v_cnt4,0
		movwf	v_pos3

		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
no_first_zero	movlw	H'12'			;space
		call	get_char
		call	i2c_send
		goto	next_one
;-------------------------------------------------------------------------
display_data	call	i2c_start
		movlw	H'02'
		call	i2c_send
		call	i2c_stop

		call	i2c_start
		movlw	H'03'
		call	i2c_send
;--------------------------------------
;--------------------------------------
		movf	v_pos0,0
		btfsc	STATUS,Z
		goto	no_first_zero
		call	get_char
		call	i2c_send

next_one	movf	v_pos1,0
		call	get_char
		btfss	v_blink,D'003'
		addlw	D'001'			;dvojbodka
		call	i2c_send

		movf	v_pos2,0
		call	get_char
		call	i2c_send

		movf	v_pos3,0
		call	get_char
		call	i2c_send
		call	i2c_stop
		return
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
clk_set_up	movlw	H'00'
		movwf	v_rtc_sec

		movf	v_rtc_hrs,0
		xorlw	H'FE'
		btfss	STATUS,Z
		goto	clk_su_loop
		clrf	v_rtc_hrs
		clrf	v_rtc_min

clk_su_loop	nop
		btfss	PORTB,D'003'
		call	inc_hrs
		btfss	PORTB,D'006'
		call	inc_min
;display the clock -----------------------------
		call	translate_clk		;(2.1) - prepare display data - part 1

		call	translate_chr2		;(2.2) - prepare display data - part 2
		call	display_data

;-----------------------------------------------
		btfss	PORTB,D'007'
		goto	clk_su_loop
;write the clock the the RTC
		call	rtc_i2c_write		;(1) - write time to the RTC chip
		call	dr2
		goto	main_loop
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
inc_hrs		incf	v_rtc_hrs,1
		movf	v_rtc_hrs,0
		xorlw	H'24'
		btfsc	STATUS,Z
		clrf	v_rtc_hrs

		movf	v_rtc_hrs,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfss	STATUS,Z
		return
		movf	v_rtc_hrs,0
		andlw	B'11110000'
		addlw	H'10'
		movwf	v_rtc_hrs
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
inc_min		incf	v_rtc_min,1
		movf	v_rtc_min,0
		xorlw	H'5A'
		btfsc	STATUS,Z
		clrf	v_rtc_min

		movf	v_rtc_min,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfss	STATUS,Z
		return
		movf	v_rtc_min,0
		andlw	B'11110000'
		addlw	H'10'
		movwf	v_rtc_min
		return
;------------------------------------------------------------------------------
clear_hrs	clrf	v_rtc_hrs
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
translate_clk	movf	v_rtc_hrs,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	no_clk

		movf	v_rtc_hrs,0
		xorlw	H'FE'
		btfsc	STATUS,Z
		goto	osc_stop

		movf	v_rtc_hrs,0
		call	calc_hex_disp
		movf	v_msb,0
		movwf	v_cnt1
		movf	v_lsb,0
		movwf	v_cnt2

		movf	v_rtc_min,0
		call	calc_hex_disp
		movf	v_msb,0
		movwf	v_cnt3
		movf	v_lsb,0
		movwf	v_cnt4

		movf	v_rtc_sec,0
		call	calc_hex_disp
		movf	v_msb,0
		movwf	v_cnt5
		movf	v_lsb,0
		movwf	v_cnt6
		return
;------------------------------------------------------------------------------
no_clk		movlw	H'10'
		movwf	v_cnt1
		movwf	v_cnt2
		movwf	v_cnt3
		movwf	v_cnt4
		movwf	v_cnt5
		movwf	v_cnt6
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
osc_stop	movlw	H'05'		;S
		movwf	v_cnt1
		movlw	H'13'		;t
		movwf	v_cnt2
		movlw	H'00'		;O
		movwf	v_cnt3
		movlw	H'14'		;P
		movwf	v_cnt4
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
calc_hex_disp	movwf	v_value
		andlw	B'00001111'
		movwf	v_lsb
		movf	v_value,0
		movwf	v_msb
		lsrf	v_msb,1
		lsrf	v_msb,1
		lsrf	v_msb,1
		lsrf	v_msb,1
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;Character Set Definition
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;		movlw	B'XABCDEFG'
;------------------------------------------------------------------------------
get_char	brw
wm_0		retlw	B'11111100'
wm_1		retlw	B'01100000'
wm_2		retlw	B'11011010'
wm_3		retlw	B'11110010'
wm_4		retlw	B'01100110'
wm_5		retlw	B'10110110'
wm_6		retlw	B'10111110'
wm_7		retlw	B'11100100'
wm_8		retlw	B'11111110'
wm_9		retlw	B'11110110'

wm_A		retlw	B'11101110'
wm_b		retlw	B'00111110'
wm_C		retlw	B'10011100'
wm_d		retlw	B'01111010'
wm_E		retlw	B'10011110'
wm_F		retlw	B'10001110'

wm_minus	retlw	B'00000010'		;10h
wm_h		retlw	B'00010111'		;11h
wm_space	retlw	B'00000000'		;12h
wm_t		retlw	B'00011110'		;13h
wm_P		retlw	B'11001110'		;14h
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
rtc_i2c_read	call	i2c_1_start
		movlw	B'11010000'		;RTC = 1101000x
		call	i2c_1_send

		movlw	B'00000000'		;memory address (= 0)
		call	i2c_1_send
		call	i2c_1_stop
;-----------------------------------------------
		call	i2c_1_start
		movlw	B'11010001'		;RTC = 1101000x
		call	i2c_1_send

		call	i2c_1_receive		;received byte - sec
		movwf	v_rtc_sec
		call	i2c_1_ack

		call	i2c_1_receive		;received byte - min
		movwf	v_rtc_min
		call	i2c_1_ack

		call	i2c_1_receive		;received byte
		movwf	v_rtc_hrs
		call	i2c_1_not_ack

		call	i2c_1_stop

		bcf	v_rtc_hrs,D'007'
		bcf	v_rtc_hrs,D'006'
		bcf	v_rtc_min,D'007'

		btfsc	v_rtc_sec,D'007'
		call	osc_stopped

		bcf	v_rtc_sec,D'007'

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
osc_stopped	movlw	H'FE'
		movwf	v_rtc_hrs
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
rtc_i2c_write	call	i2c_1_start
		movlw	B'11010000'		;RTC = 1101000x
		call	i2c_1_send

		movlw	B'00000000'		;memory address (= 0)
		call	i2c_1_send
;-----------------------------------------------
		movf	v_rtc_sec,0
		call	i2c_1_send

		movf	v_rtc_min,0
		call	i2c_1_send

		movf	v_rtc_hrs,0
		call	i2c_1_send

		call	i2c_1_stop
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;******************************************************************************
;******************************************************************************
		END